@@ -13,11 +13,9 @@ from trytond.pyson import If, In, Eval, Bool
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
-__all__ = ['PurchaseRequest',
-    'CreatePurchaseAskParty', 'CreatePurchase',
-    'HandlePurchaseCancellationException',
-    'HandlePurchaseCancellationExceptionStart',
-    ]
+from trytond.modules.company.model import (
+    employee_field, set_employee, reset_employee)
+from trytond.modules.product import round_price
 
 STATES = {
     'readonly': Eval('state') != 'draft',
@@ -40,7 +38,9 @@ class PurchaseRequest(ModelSQL, ModelView):
     uom = fields.Many2One('product.uom', 'UOM', select=True,
         ondelete='RESTRICT',
         domain=[
-            ('category', '=', Eval('product_uom_category')),
+            If(Bool(Eval('product_uom_category')),
+                ('category', '=', Eval('product_uom_category')),
+                ('category', '!=', -1)),
             ],
         states={
             'required': Bool(Eval('product')),
@@ -82,6 +82,9 @@ class PurchaseRequest(ModelSQL, ModelView):
             ])
     origin = fields.Reference('Origin', selection='get_origin', readonly=True)
     exception_ignored = fields.Boolean('Ignored Exception')
+
+    purchased_by = employee_field(
+        "Purchased By", states=['purchased', 'done', 'cancel', 'exception'])
     state = fields.Selection([
             ('purchased', "Purchased"),
             ('done', "Done"),
@@ -162,15 +165,20 @@ class PurchaseRequest(ModelSQL, ModelView):
         tablehandler.not_null_action('origin', action='remove')
 
     def get_rec_name(self, name):
+        pool = Pool()
+        Lang = pool.get('ir.lang')
         if self.product:
-            rec_name = self.product.name
+            lang = Lang.get()
+            rec_name = (lang.format(
+                    '%.*f', (self.uom.digits, self.quantity))
+                + '%s %s' % (self.uom.symbol, self.product.name))
         elif self.description:
             rec_name = self.description.splitlines()[0]
         else:
             rec_name = str(self.id)
 
         if self.warehouse:
-            return "%s@%s" % (rec_name, self.warehouse.name)
+            return "%s @% s" % (rec_name, self.warehouse.name)
         else:
             return rec_name
 
@@ -230,6 +238,16 @@ class PurchaseRequest(ModelSQL, ModelView):
                 request.state = state
         cls.save(requests)
 
+    @classmethod
+    @set_employee('purchased_by')
+    def set_purchased(cls, requests):
+        cls.update_state(requests)
+
+    @classmethod
+    @reset_employee('purchased_by')
+    def reset_purchased(cls, requests):
+        cls.update_state(requests)
+
     def get_warehouse_required(self, name):
         return self.product and self.product.type in ('goods', 'assets')
 
@@ -453,7 +471,7 @@ class CreatePurchase(Wizard):
                 lines.append(line)
         Purchase.save(purchases)
         Line.save(lines)
-        Request.update_state(requests)
+        Request.set_purchased(requests)
         return 'end'
 
     @classmethod
@@ -462,8 +480,7 @@ class CreatePurchase(Wizard):
         Line = pool.get('purchase.line')
 
         line = Line()
-        line.unit_price = Decimal(0).quantize(
-            Decimal(1) / 10 ** Line.unit_price.digits[1])
+        line.unit_price = round_price(Decimal(0))
         for f, v in key:
             setattr(line, f, v)
         line.quantity = sum(r.quantity for r in requests)
@@ -497,7 +514,7 @@ class HandlePurchaseCancellationException(Wizard):
         requests = PurchaseRequest.browse(Transaction().context['active_ids'])
         for request in requests:
             request.purchase_line = None
-        PurchaseRequest.update_state(requests)
+        PurchaseRequest.reset_purchased(requests)
         return 'end'
 
     def transition_cancel_request(self):
